@@ -11,7 +11,7 @@ from pychess.Utils.book import getOpenings
 from pychess.Utils.const import NORMALCHESS, FEN_START, BLACK, FISCHERRANDOMCHESS, \
     CRAZYHOUSECHESS, WILDCASTLESHUFFLECHESS, LOSERSCHESS, SUICIDECHESS, ATOMICCHESS, \
     THREECHECKCHESS, KINGOFTHEHILLCHESS, ASEANCHESS, MAKRUKCHESS, CAMBODIANCHESS, \
-    SITTUYINCHESS, WHITE
+    SITTUYINCHESS, GIVEAWAYCHESS, HORDECHESS, WHITE
 from pychess.Utils.lutils.Benchmark import benchmark
 from pychess.Utils.lutils.perft import perft
 from pychess.Utils.lutils.LBoard import LBoard
@@ -21,6 +21,7 @@ from pychess.Utils.lutils.lmove import parseSAN, parseAny, toSAN, ParsingError
 from pychess.Utils.lutils.lmovegen import genAllMoves, genCaptures, genCheckEvasions
 from pychess.Utils.lutils.validator import validateMove
 from pychess.System.Log import log
+from pychess.Variants.horde import HORDESTART
 from pychess.Variants.asean import ASEANSTART, MAKRUKSTART, KAMBODIANSTART, SITTUYINSTART
 
 if sys.platform != "win32":
@@ -30,6 +31,10 @@ if sys.platform != "win32":
 ASCII = sys.platform == "win32"
 
 
+def get_input():
+    return input()
+
+
 class PyChessCECP(PyChess):
     def __init__(self):
         PyChess.__init__(self)
@@ -55,7 +60,8 @@ class PyChessCECP(PyChess):
             "reuse": 1,
             "analyze": 1,
             "myname": "PyChess %s" % pychess.VERSION,
-            "variants": "normal,wildcastle,nocastle,fischerandom,crazyhouse,losers,suicide,atomic," +
+            "variants": "normal,wildcastle,nocastle,fischerandom,crazyhouse," +
+                        "losers,suicide,giveaway,horde,atomic," +
                         "kingofthehill,3check,asean,cambodian,makruk,sittuyin",
             "colors": 0,
             "ics": 0,
@@ -84,7 +90,7 @@ class PyChessCECP(PyChess):
     def run(self):
         while True:
             try:
-                line = input()
+                line = get_input()
             except EOFError:
                 line = "quit"
             lines = line.split()
@@ -138,6 +144,8 @@ class PyChessCECP(PyChess):
                             self.board.variant = LOSERSCHESS
                         elif lines[1] == "suicide":
                             self.board.variant = SUICIDECHESS
+                        elif lines[1] == "giveaway":
+                            self.board.variant = GIVEAWAYCHESS
                         elif lines[1] == "atomic":
                             self.board.variant = ATOMICCHESS
                             self.board.iniAtomic()
@@ -147,6 +155,9 @@ class PyChessCECP(PyChess):
                             self.board.variant = KINGOFTHEHILLCHESS
                             self.print("setup (PNBRQKpnbrqk) 8x8+0_fairy %s" %
                                        FEN_START)
+                        elif lines[1] == "horde":
+                            self.board = LBoard(HORDECHESS)
+                            self.board.applyFen(HORDESTART)
                         elif lines[1] == "asean":
                             self.board = LBoard(ASEANCHESS)
                             self.board.applyFen(ASEANSTART)
@@ -385,7 +396,10 @@ class PyChessCECP(PyChess):
                                 for move in genCheckEvasions(self.board)])
 
                 elif lines[0] == "benchmark":
-                    benchmark()
+                    if len(lines) > 1:
+                        benchmark(int(lines[1]))
+                    else:
+                        benchmark()
 
                 elif lines[0] == "profile":
                     if len(lines) > 1:
@@ -403,11 +417,14 @@ class PyChessCECP(PyChess):
                     else:
                         self.print("Error (arguments must be integer")
 
+                elif lines[0] == "stop_unittest":
+                    break
+
                 elif len(lines) == 1:
                     # A GUI without usermove support might try to send a move.
                     try:
                         move = parseAny(self.board, line)
-                    except:
+                    except ParsingError:
                         self.print("Error (unknown command): %s" % line)
                         continue
                     if not validateMove(self.board, move):
