@@ -1,8 +1,5 @@
-
 import asyncio
 import collections
-from copy import copy
-
 
 from pychess.Utils import wait_signal
 from pychess.Utils.Move import parseAny
@@ -44,9 +41,9 @@ class UCIEngine(ProtocolEngine):
         self.ignoreNext = False
         self.waitingForMove = False
         self.needBestmove = False
+        self.bestmove_event = asyncio.Event()
         self.readyForStop = False  # keeps track of whether we already sent a 'stop' command
-        self.multipvSetting = conf.get("multipv", 1
-                                       )  # MultiPV option sent to the engine
+        self.multipvSetting = 1  # MultiPV option sent to the engine
         self.multipvExpected = 1  # Number of PVs expected (limited by number of legal moves)
         self.commands = collections.deque()
 
@@ -81,7 +78,7 @@ class UCIEngine(ProtocolEngine):
         except asyncio.TimeoutError:
             log.warning("Got timeout error", extra={"task": self.defname})
             raise PlayerIsDead
-        except:
+        except Exception:
             log.warning("Unknown error", extra={"task": self.defname})
             raise PlayerIsDead
         else:
@@ -97,9 +94,6 @@ class UCIEngine(ProtocolEngine):
             if self.hasOption("Ponder"):
                 self.setOption('Ponder', False)
 
-            if self.hasOption("MultiPV") and self.multipvSetting > 1:
-                self.setOption('MultiPV', self.multipvSetting)
-
         for option, value in self.optionsToBeSent.items():
             if isinstance(value, bool):
                 value = str(value).lower()
@@ -113,7 +107,7 @@ class UCIEngine(ProtocolEngine):
         self.queue.put_nowait("ready")
         self._newGame()
 
-        if self.mode in (ANALYZING, INVERSE_ANALYZING):
+        if self.isAnalyzing():
             self._searchNow()
 
     # Ending the game
@@ -198,23 +192,41 @@ class UCIEngine(ProtocolEngine):
                                         model.boards[0:ply]):
             self._recordMove(board1, move, board2)
 
-    def setBoard(self, board):
+    def setBoard(self, board, search=True):
         log.debug("setBoardAtPly: board=%s" % board,
                   extra={"task": self.defname})
-        self._recordMove(board, None, None)
-
         if not self.readyMoves:
             return
-        self._searchNow()
+
+        @asyncio.coroutine
+        def coro():
+            if self.needBestmove:
+                self.bestmove_event.clear()
+                print("stop", file=self.engine)
+                yield from self.bestmove_event.wait()
+
+            self._recordMove(board, None, None)
+            if search:
+                self._searchNow()
+        asyncio.async(coro())
 
     def putMove(self, board1, move, board2):
         log.debug("putMove: board1=%s move=%s board2=%s self.board=%s" % (
             board1, move, board2, self.board), extra={"task": self.defname})
-        self._recordMove(board1, move, board2)
-
         if not self.readyMoves:
             return
-        self._searchNow()
+
+        @asyncio.coroutine
+        def coro():
+            if self.needBestmove:
+                self.bestmove_event.clear()
+                print("stop", file=self.engine)
+                yield from self.bestmove_event.wait()
+
+            self._recordMove(board1, move, board2)
+            if not self.analyzing_paused:
+                self._searchNow()
+        asyncio.async(coro())
 
     @asyncio.coroutine
     def makeMove(self, board1, move, board2):
@@ -239,7 +251,7 @@ class UCIEngine(ProtocolEngine):
             return_queue = yield from self.queue.get()
             if return_queue == "invalid":
                 raise InvalidMove
-            if return_queue == "del":
+            if return_queue == "del" or return_queue == "die":
                 raise PlayerIsDead
             if return_queue == "int":
                 self.pondermove = None
@@ -313,12 +325,21 @@ class UCIEngine(ProtocolEngine):
 
     def pause(self):
         log.debug("pause: self=%s" % self, extra={"task": self.defname})
-        self.engine.pause()
+        if self.isAnalyzing():
+            print("stop", file=self.engine)
+            self.readyForStop = False
+            self.analyzing_paused = True
+        else:
+            self.engine.pause()
         return
 
     def resume(self):
         log.debug("resume: self=%s" % self, extra={"task": self.defname})
-        self.engine.resume()
+        if self.isAnalyzing():
+            self._searchNow()
+            self.analyzing_paused = False
+        else:
+            self.engine.resume()
         return
 
     def hurry(self):
@@ -376,22 +397,16 @@ class UCIEngine(ProtocolEngine):
         """ Set an option, which will be sent to the engine, after the
             'readyForOptions' signal has passed.
             If you want to know the possible options, you should go to
-            engineDiscoverer or use the getOption, getOptions and hasOption
-            methods, while you are in your 'readyForOptions' signal handler """
+            engineDiscoverer or use the hasOption method
+            while you are in your 'readyForOptions' signal handler """
         if self.readyMoves:
             log.warning(
                 "Options set after 'readyok' are not sent to the engine",
                 extra={"task": self.defname})
         self.optionsToBeSent[key] = value
         self.ponderOn = key == "Ponder" and value is True
-
-    def getOption(self, option):
-        if option in self.options:
-            return self.options[option]["default"]
-        return None
-
-    def getOptions(self):
-        return copy(self.options)
+        if key == "MultiPV":
+            self.multipvSetting = int(value)
 
     def hasOption(self, key):
         return key in self.options
@@ -430,16 +445,18 @@ class UCIEngine(ProtocolEngine):
                     # Many engines don't like positions able to take down enemy
                     # king. Therefore we just return the "kill king" move
                     # automaticaly
-                    self.emit("analyze", [([toAN(
-                        self.board, getMoveKillingKing(self.board))], MATE_VALUE - 1, "")])
+                    self.emit("analyze", [(self.board.ply, [toAN(
+                        self.board, getMoveKillingKing(self.board))], MATE_VALUE - 1, "", "")])
                     return
                 commands.append("position fen %s" % self.board.asFen())
             else:
                 commands.append("position %s" % self.uciPosition)
 
-            # commands.append("go infinite")
-            move_time = int(conf.get("max_analysis_spin", 3)) * 1000
-            commands.append("go movetime %s" % move_time)
+            if conf.get("infinite_analysis", False):
+                commands.append("go infinite")
+            else:
+                move_time = int(conf.get("max_analysis_spin", 3)) * 1000
+                commands.append("go movetime %s" % move_time)
 
         if self.hasOption("MultiPV") and self.multipvSetting > 1:
             self.multipvExpected = min(self.multipvSetting,
@@ -526,6 +543,7 @@ class UCIEngine(ProtocolEngine):
                 # A Move
                 if self.mode == NORMAL and parts[0] == "bestmove":
                     self.needBestmove = False
+                    self.bestmove_event.set()
                     self.__sendQueuedGo()
 
                     if self.ignoreNext:
@@ -548,6 +566,10 @@ class UCIEngine(ProtocolEngine):
                         move = parseAny(self.board, movestr)
                     except ParsingError:
                         self.invalid_move = movestr
+                        log.info(
+                            "__parseLine: ParsingError engine move: %s %s"
+                            % (movestr, self.board),
+                            extra={"task": self.defname})
                         self.end(WHITEWON if self.board.color == BLACK else
                                  BLACKWON, WON_ADJUDICATION)
                         continue
@@ -612,9 +634,13 @@ class UCIEngine(ProtocolEngine):
                     else:
                         depth = ""
 
-                    if multipv <= len(self.analysis):
-                        self.analysis[multipv - 1] = (movstrs, score, depth)
+                    if "nps" in parts:
+                        nps = parts[parts.index("nps") + 1]
+                    else:
+                        nps = ""
 
+                    if multipv <= len(self.analysis):
+                        self.analysis[multipv - 1] = (self.board.ply, movstrs, score, depth, nps)
                     self.emit("analyze", self.analysis)
                     continue
 
@@ -623,6 +649,7 @@ class UCIEngine(ProtocolEngine):
                     log.debug("__parseLine: processing analyzer bestmove='%s'" % line.strip(),
                               extra={"task": self.defname})
                     self.needBestmove = False
+                    self.bestmove_event.set()
                     self.__sendQueuedGo(sendlast=True)
                     continue
 
@@ -669,6 +696,18 @@ class UCIEngine(ProtocolEngine):
 
     #    Info
 
+    def getAnalysisLines(self):
+        try:
+            return int(self.optionsToBeSent["MultiPV"])
+        except (KeyError, ValueError):
+            return 1  # Engine does not support the MultiPV option
+
+    def minAnalysisLines(self):
+        try:
+            return int(self.options["MultiPV"]["min"])
+        except (KeyError, ValueError):
+            return 1  # Engine does not support the MultiPV option
+
     def maxAnalysisLines(self):
         try:
             return int(self.options["MultiPV"]["max"])
@@ -676,16 +715,13 @@ class UCIEngine(ProtocolEngine):
             return 1  # Engine does not support the MultiPV option
 
     def requestMultiPV(self, n):
-        multipvMax = self.maxAnalysisLines()
-        n = min(n, multipvMax)
-
+        n = min(n, self.maxAnalysisLines())
+        n = max(n, self.minAnalysisLines())
         if n != self.multipvSetting:
-            conf.set("multipv", n)
             self.multipvSetting = n
             print("stop", file=self.engine)
             print("setoption name MultiPV value", n, file=self.engine)
             self._searchNow()
-
         return n
 
     def __repr__(self):
