@@ -22,7 +22,10 @@ from pychess.external.scoutfish import Scoutfish
 from pychess.external.chess_db import Parser
 
 from pychess.Utils.const import WHITE, BLACK, reprResult, FEN_START, FEN_EMPTY, \
-    WON_RESIGN, DRAW, BLACKWON, WHITEWON, NORMALCHESS, DRAW_AGREE, FIRST_PAGE, PREV_PAGE, NEXT_PAGE
+    WON_RESIGN, DRAW, BLACKWON, WHITEWON, NORMALCHESS, DRAW_AGREE, FIRST_PAGE, PREV_PAGE, NEXT_PAGE, \
+    ABORTED_REASONS, ADJOURNED_REASONS, WON_CALLFLAG, DRAW_ADJUDICATION, WON_ADJUDICATION, \
+    WHITE_ENGINE_DIED, BLACK_ENGINE_DIED
+
 from pychess.System import conf
 from pychess.System.Log import log
 from pychess.System.prefix import getEngineDataPrefix
@@ -30,6 +33,7 @@ from pychess.Utils.lutils.LBoard import LBoard
 from pychess.Utils.GameModel import GameModel
 from pychess.Utils.lutils.lmove import toSAN, parseSAN, ParsingError
 from pychess.Utils.Move import Move
+from pychess.Utils.elo import get_elo_rating_change_pgn
 from pychess.Utils.logic import getStatus
 from pychess.Utils.lutils.ldata import MATE_VALUE
 from pychess.Variants import name2variant, NormalBoard, variants
@@ -58,11 +62,11 @@ pattern = re.compile(r"""
     |(\))                # variation end
     |(\*|1-0|0-1|1/2)    # result (spec requires 1/2-1/2 for draw, but we want to tolerate simple 1/2 too)
     |(
-    ([a-hKQRBNMSF][a-hxKQRBNMSF1-8+#=\-]{1,6}
+    ([a-hKkQqRrBNnMmSsF][a-hxKQRBNMSF1-8+#=\-]{1,6}
     |[PNBRQMSFK]@[a-h][1-8][+#]?  # drop move
-    |o\-o(?:\-o)?
-    |O\-O(?:\-O)?
-    |0\-0(?:\-0)?
+    |o\-?o(?:\-?o)?      # castling notation using letter 'o' with or without '-'
+    |O\-?O(?:\-?O)?      # castling notation using letter 'O' with or without '-'
+    |0\-0(?:\-0)?        # castling notation using zero with required '-'
     |\-\-)               # non standard '--' is used for null move inside variations
     ([\?!]{1,2})*
     )    # move (full, count, move with ?!, ?!)
@@ -119,30 +123,55 @@ def parseTimeControlTag(tag):
     match = re.match("(\d+)(?:\+(\d+))?", tag)
     if match:
         secs, gain = match.groups()
-        return int(secs), int(gain) if gain is not None else 0
+        return int(secs), int(gain) if gain is not None else 0, 0
+    else:
+        match = re.match("(\d+)(?:\/(\d+))?", tag)
+        if match:
+            moves, secs = match.groups()
+            return int(secs), 0, int(moves)
+        else:
+            return None
 
 
 def save(handle, model, position=None):
     """ Saves the game from GameModel to .pgn """
 
+    # Header
     status = "%s" % reprResult[model.status]
 
-    print('[Event "%s"]' % model.tags["Event"], file=handle)
-    print('[Site "%s"]' % model.tags["Site"], file=handle)
+    print('[Event "%s"]' % model.getTagExport("Event", ""), file=handle)
+    print('[Site "%s"]' % model.getTagExport("Site", ""), file=handle)
     print('[Date "%04d.%02d.%02d"]' %
           (int(model.tags["Year"]), int(model.tags["Month"]), int(model.tags["Day"])), file=handle)
-    print('[Round "%s"]' % model.tags["Round"], file=handle)
+    print('[Round "%s"]' % model.getTagExport("Round", "?"), file=handle)
     print('[White "%s"]' % repr(model.players[WHITE]), file=handle)
     print('[Black "%s"]' % repr(model.players[BLACK]), file=handle)
     print('[Result "%s"]' % status, file=handle)
-    if "ECO" in model.tags:
-        print('[ECO "%s"]' % model.tags["ECO"], file=handle)
-    if "WhiteElo" in model.tags:
-        print('[WhiteElo "%s"]' % model.tags["WhiteElo"], file=handle)
-    if "BlackElo" in model.tags:
-        print('[BlackElo "%s"]' % model.tags["BlackElo"], file=handle)
-    if "TimeControl" in model.tags:
-        print('[TimeControl "%s"]' % model.tags["TimeControl"], file=handle)
+    tag = model.getTagExport("ECO", "")
+    if tag != "":
+        print('[ECO "%s"]' % tag, file=handle)
+        tag = model.getTagExport("Opening", "")
+        if tag != "":
+            print('[Opening "%s"]' % tag, file=handle)  # Unofficial
+            tag = model.getTagExport("Variation", "")
+            if tag != "":
+                print('[Variation "%s"]' % tag, file=handle)  # Unofficial
+    welo = model.getTagExport("WhiteElo", "")
+    if welo != "":
+        print('[WhiteElo "%s"]' % welo, file=handle)
+    belo = model.getTagExport("BlackElo", "")
+    if belo != "":
+        print('[BlackElo "%s"]' % belo, file=handle)
+    if welo != "" and belo != "" and conf.get("saveRatingChange", False):
+        diff = str(get_elo_rating_change_pgn(model, WHITE))
+        if diff != "":
+            print('[WhiteRatingDiff "%s"]' % diff, file=handle)  # Unofficial
+        diff = str(get_elo_rating_change_pgn(model, BLACK))
+        if diff != "":
+            print('[BlackRatingDiff "%s"]' % diff, file=handle)  # Unofficial
+    tag = model.getTagExport("TimeControl", "")
+    if tag != "":
+        print('[TimeControl "%s"]' % tag, file=handle)
     if model.timed:
         print('[WhiteClock "%s"]' %
               msToClockTimeTag(int(model.timemodel.getPlayerTime(WHITE) * 1000)), file=handle)
@@ -157,20 +186,74 @@ def save(handle, model, position=None):
         print('[SetUp "1"]', file=handle)
         print('[FEN "%s"]' % model.boards[0].asFen(), file=handle)
     print('[PlyCount "%s"]' % (model.ply - model.lowply), file=handle)
-    if "Annotator" in model.tags:
-        print('[Annotator "%s"]' % model.tags["Annotator"], file=handle)
-    print("", file=handle)
+    tag = model.getTagExport("Annotator", "")
+    if tag != "":
+        print('[Annotator "%s"]' % tag, file=handle)
+    if model.reason == WON_CALLFLAG:
+        termination = "time forfeit"
+    elif model.reason == WON_ADJUDICATION and model.isEngine2EngineGame():
+        termination = "rules infraction"
+    elif model.reason in (DRAW_ADJUDICATION, WON_ADJUDICATION):
+        termination = "adjudication"
+    elif model.reason == WHITE_ENGINE_DIED:
+        termination = "white engine died"
+    elif model.reason == BLACK_ENGINE_DIED:
+        termination = "black engine died"
+    elif model.reason in ABORTED_REASONS:
+        termination = "abandoned"
+    elif model.reason in ADJOURNED_REASONS:
+        termination = "unterminated"
+    else:
+        termination = "normal"
+    print('[Termination "%s"]' % termination, file=handle)
 
     save_emt = conf.get("saveEmt", False)
     save_eval = conf.get("saveEval", False)
 
+    # Discovery of the moves and comments
     result = []
     walk(model.boards[0].board, result, model, save_emt, save_eval)
 
-    result = " ".join(result)
-    result = wrap(result, 80)
-    print(result, status, file=handle)
-    print("", file=handle)
+    # Alignment of the fetched elements
+    indented = conf.get("indentPgn", False)
+    buffer = ""
+    depth = 0
+    crlf = False
+    for text in result:
+        # De/Indentation
+        crlf = (buffer[-1:] if len(buffer) > 0 else "") in ["\r", "\n"]
+        if text == "(":
+            depth += 1
+            if indented and not crlf:
+                buffer += os.linesep
+                crlf = True
+        # Space between each term
+        last = buffer[-1:] if len(buffer) > 0 else ""
+        crlf = last in ["\r", "\n"]
+        if not crlf and last != " " and last != "\t" and last != "(" and not text.startswith("\r") and not text.startswith("\n") and text != ")" and len(buffer) > 0:
+            buffer += " "
+        # New line for a new main move
+        if len(buffer) == 0 or (indented and depth == 0 and last != "\r" and last != "\n" and re.match("^[0-9]+\.", text) is not None):
+            buffer += os.linesep
+            crlf = True
+        # Alignment
+        if crlf and depth > 0:
+            for j in range(0, depth):
+                buffer += "    "
+        # Term
+        buffer += text
+        if indented and text == ")":
+            buffer += os.linesep
+            crlf = True
+            depth -= 1
+
+    # Status of the game
+    buffer += ("" if crlf else (os.linesep if indented else " ")) + status
+    if not indented:
+        buffer = wrap(buffer, 80)
+
+    # Final
+    print(buffer, "", file=handle)
     output = handle.getvalue() if isinstance(handle, StringIO) else ""
     handle.close()
     return output
@@ -185,14 +268,6 @@ def walk(node, result, model, save_emt=False, save_eval=False, vari=False):
        node - list (a tree of lboards created by the pgn parser)
        result - str (movetext strings)"""
 
-    def store(text):
-        if len(result) > 1 and result[-1] == "(":
-            result[-1] = "(%s" % text
-        elif text == ")":
-            result[-1] = "%s)" % result[-1]
-        else:
-            result.append(text)
-
     while True:
         if node is None:
             break
@@ -201,7 +276,7 @@ def walk(node, result, model, save_emt=False, save_eval=False, vari=False):
         if node.prev is None:
             for child in node.children:
                 if isinstance(child, str):
-                    store("{%s}" % child)
+                    result.append("{%s}%s" % (child, os.linesep))
             node = node.next
             continue
 
@@ -210,9 +285,9 @@ def walk(node, result, model, save_emt=False, save_eval=False, vari=False):
                                "TimeControl" in model.tags)
         if movecount is not None:
             if movecount:
-                store(movecount)
+                result.append(movecount)
             move = node.lastMove
-            store(toSAN(node.prev, move))
+            result.append(toSAN(node.prev, move))
             if (save_emt or save_eval) and not vari:
                 emt_eval = ""
                 if "TimeControl" in model.tags and save_emt:
@@ -225,40 +300,40 @@ def walk(node, result, model, save_emt=False, save_eval=False, vari=False):
                         score = -score
                     emt_eval += "[%%eval %0.2f/%s]" % (score / 100.0, depth)
                 if emt_eval:
-                    store("{%s}" % emt_eval)
+                    result.append("{%s}" % emt_eval)
 
         for nag in node.nags:
             if nag:
-                store(nag)
+                result.append(nag)
 
         for child in node.children:
             if isinstance(child, str):
                 child = re.sub("\[%.*?\]", "", child)
                 # comment
                 if child:
-                    store("{%s}" % child)
+                    result.append("{%s}" % child)
             else:
                 # variations
                 if node.fen_was_applied:
-                    store("(")
+                    result.append("(")
                     walk(child[0],
                          result,
                          model,
                          save_emt,
                          save_eval,
                          vari=True)
-                    store(")")
+                    result.append(")")
                     # variation after last played move is not valid pgn
                     # but we will save it as in comment
                 else:
-                    store("{Analyzer's primary variation:")
+                    result.append("{%s:" % _("Analyzer's primary variation"))
                     walk(child[0],
                          result,
                          model,
                          save_emt,
                          save_eval,
                          vari=True)
-                    store("}")
+                    result.append("}")
 
         if node.next:
             node = node.next
@@ -321,7 +396,7 @@ chess_db_path = shutil.which(parser, mode=os.X_OK, path=altpath)
 
 
 class PGNFile(ChessFile):
-    def __init__(self, handle, progressbar):
+    def __init__(self, handle, progressbar=None):
         ChessFile.__init__(self, handle)
         self.handle = handle
         self.progressbar = progressbar
@@ -364,6 +439,10 @@ class PGNFile(ChessFile):
         return os.path.getsize(self.path)
     size = property(get_size)
 
+    def close(self):
+        self.tag_database.close()
+        ChessFile.close(self)
+
     def init_tag_database(self, importer):
         """ Create/open .sqlite database of game header tags """
         # Import .pgn header tags to .sqlite database
@@ -371,7 +450,8 @@ class PGNFile(ChessFile):
         if size > 0 and self.tag_database.count == 0:
             if size > 10000000:
                 drop_indexes(self.engine)
-            GLib.idle_add(self.progressbar.set_text, _("Importing game headers..."))
+            if self.progressbar is not None:
+                GLib.idle_add(self.progressbar.set_text, _("Importing game headers..."))
             importer.initialize()
             importer.do_import(self.path, progressbar=self.progressbar)
             if size > 10000000 and not importer.cancel:
@@ -601,19 +681,19 @@ class PGNFile(ChessFile):
             rec = self.games[0]
             game_date = rec["Date"]
             result = rec["Result"]
-            variant = rec["Variant"]
+            variant = rec["Variant"].capitalize()
         else:
             game_date = self.get_date(rec)
             result = reprResult[rec["Result"]]
             variant = self.get_variant(rec)
 
         # the seven mandatory PGN headers
-        model.tags['Event'] = rec["Event"]
-        model.tags['Site'] = rec["Site"]
+        model.tags['Event'] = rec["Event"] if rec["Event"] is not None else ""
+        model.tags['Site'] = rec["Site"] if rec["Site"] is not None else ""
         model.tags['Date'] = game_date
         model.tags['Round'] = rec["Round"]
-        model.tags['White'] = rec["White"]
-        model.tags['Black'] = rec["Black"]
+        model.tags['White'] = rec["White"] if rec["White"] is not None else "?"
+        model.tags['Black'] = rec["Black"] if rec["Black"] is not None else "?"
         model.tags['Result'] = result
 
         if model.tags['Date']:
@@ -637,26 +717,29 @@ class PGNFile(ChessFile):
             model.info = self.tag_database.get_info(rec)
 
         if model.tags['TimeControl']:
-            secs, gain = parseTimeControlTag(model.tags['TimeControl'])
-            model.timed = True
-            model.timemodel.secs = secs
-            model.timemodel.gain = gain
-            model.timemodel.minutes = secs / 60
-            for tag, color in (('WhiteClock', WHITE), ('BlackClock', BLACK)):
-                if hasattr(rec, tag):
-                    try:
-                        millisec = parseClockTimeTag(rec[tag])
-                        # We need to fix when FICS reports negative clock time like this
-                        # [TimeControl "180+0"]
-                        # [WhiteClock "0:00:15.867"]
-                        # [BlackClock "23:59:58.820"]
-                        start_sec = (
-                            millisec - 24 * 60 * 60 * 1000
-                        ) / 1000. if millisec > 23 * 60 * 60 * 1000 else millisec / 1000.
-                        model.timemodel.intervals[color][0] = start_sec
-                    except ValueError:
-                        raise LoadingError(
-                            "Error parsing '%s'" % tag)
+            tc = parseTimeControlTag(model.tags['TimeControl'])
+            if tc is not None:
+                secs, gain, moves = tc
+                model.timed = True
+                model.timemodel.secs = secs
+                model.timemodel.gain = gain
+                model.timemodel.minutes = secs / 60
+                model.timemodel.moves = moves
+                for tag, color in (('WhiteClock', WHITE), ('BlackClock', BLACK)):
+                    if hasattr(rec, tag):
+                        try:
+                            millisec = parseClockTimeTag(rec[tag])
+                            # We need to fix when FICS reports negative clock time like this
+                            # [TimeControl "180+0"]
+                            # [WhiteClock "0:00:15.867"]
+                            # [BlackClock "23:59:58.820"]
+                            start_sec = (
+                                millisec - 24 * 60 * 60 * 1000
+                            ) / 1000. if millisec > 23 * 60 * 60 * 1000 else millisec / 1000.
+                            model.timemodel.intervals[color][0] = start_sec
+                        except ValueError:
+                            raise LoadingError(
+                                "Error parsing '%s'" % tag)
 
         fenstr = rec["FEN"]
 
@@ -699,7 +782,6 @@ class PGNFile(ChessFile):
 
         self.error = None
         movetext = self.get_movetext(rec)
-
         boards = self.parse_movetext(movetext, boards[0], position)
 
         # The parser built a tree of lboard objects, now we have to
@@ -720,7 +802,7 @@ class PGNFile(ChessFile):
                 move = Move(node.lastMove)
                 try:
                     board = node.prev.pieceBoard.move(move, lboard=node)
-                except:
+                except Exception:
                     raise LoadingError(
                         _("Invalid move."),
                         "%s%s" % (move_count(node, black_periods=True), move))
@@ -747,7 +829,7 @@ class PGNFile(ChessFile):
         # which will be model.variations[0] when we are in the mainline.
         walk(model, boards[0], [])
         model.boards = model.variations[0]
-        self.has_emt = self.has_emt and "TimeControl" in model.tags
+        self.has_emt = self.has_emt and model.timed
         if self.has_emt or self.has_eval:
             if self.has_emt:
                 blacks = len(model.moves) // 2
@@ -757,7 +839,6 @@ class PGNFile(ChessFile):
                     [model.timemodel.intervals[0][0]] * (whites + 1),
                     [model.timemodel.intervals[1][0]] * (blacks + 1),
                 ]
-                secs, gain = parseTimeControlTag(model.tags['TimeControl'])
                 model.timemodel.intervals[0][0] = secs
                 model.timemodel.intervals[1][0] = secs
             for ply, board in enumerate(boards):
@@ -888,7 +969,7 @@ class PGNFile(ChessFile):
                         errstr2 = _("The move failed because %s.") % reason
                         self.error = LoadingError(errstr1, errstr2)
                         break
-                    except:
+                    except Exception:
                         ply = last_board.plyCount
                         if ply % 2 == 0:
                             moveno = "%d." % (ply // 2 + 1)
@@ -950,23 +1031,35 @@ class PGNFile(ChessFile):
 
     def get_movetext(self, rec):
         self.handle.seek(rec["Offset"])
+        in_comment = False
         lines = []
         line = self.handle.readline()
         if not line.strip():
             line = self.handle.readline()
 
         while line:
-            if line.startswith("["):
+            # escape non-PGN data line
+            if line.startswith("%"):
                 line = self.handle.readline()
-            elif line.startswith("%"):
+                continue
+
+            # header tag line
+            if not in_comment and line.startswith("["):
                 line = self.handle.readline()
-            elif line.strip():
+                continue
+
+            # update in_comment state
+            if (not in_comment and "{" in line) or (in_comment and "}" in line):
+                in_comment = line.rfind("{") > line.rfind("}")
+
+            if line.strip():
                 lines.append(line)
                 line = self.handle.readline()
             elif len(lines) == 0:
                 line = self.handle.readline()
             else:
                 break
+
         return "".join(lines)
 
     def get_variant(self, rec):
