@@ -8,7 +8,7 @@ from io import StringIO
 from gi.repository import GObject
 
 from pychess.Savers.ChessFile import LoadingError
-from pychess.Players.Player import PlayerIsDead, TurnInterrupt, InvalidMove
+from pychess.Players.Player import PlayerIsDead, PassInterrupt, TurnInterrupt, InvalidMove, GameEnded
 from pychess.System import conf
 from pychess.System.protoopen import protoopen, protosave
 from pychess.System.Log import log
@@ -148,9 +148,10 @@ class GameModel(GObject.GObject):
         self.timed = self.timemodel.minutes != 0 or self.timemodel.gain != 0
         if self.timed:
             self.zero_reached_cid = self.timemodel.connect('zero_reached', self.zero_reached)
-
-            self.tags["TimeControl"] = \
-                "%d+%d" % (self.timemodel.minutes * 60, self.timemodel.gain)
+            if self.timemodel.moves == 0:
+                self.tags["TimeControl"] = "%d+%d" % (self.timemodel.minutes * 60, self.timemodel.gain)
+            else:
+                self.tags["TimeControl"] = "%d/%d" % (self.timemodel.moves, self.timemodel.minutes * 60)
             # Notice: tags["WhiteClock"] and tags["BlackClock"] are never set
             # on the gamemodel, but simply written or read during saving/
             # loading from pgn. If you want to know the time left for a player,
@@ -173,7 +174,7 @@ class GameModel(GObject.GObject):
         self.undoQueue = Queue()
 
     def zero_reached(self, timemodel, color):
-        if conf.get('autoCallFlag', False) and self.players[1 - color].__type__ == ARTIFICIAL:
+        if conf.get('autoCallFlag', True) and self.players[1 - color].__type__ == ARTIFICIAL:
             if self.status == RUNNING and timemodel.getPlayerTime(color) <= 0:
                 log.info(
                     'Automatically sending flag call on behalf of player %s.' %
@@ -225,6 +226,19 @@ class GameModel(GObject.GObject):
         log.debug("GameModel.setPlayers: <- emit players_changed")
         log.debug("GameModel.setPlayers: returning")
 
+    def getTag(self, name, defaultValue):
+        if name in self.tags and self.tags[name] is not None:
+            return self.tags[name]
+        else:
+            return defaultValue
+
+    def getTagExport(self, name, defaultValue):
+        val = self.getTag(name, defaultValue)
+        if type(val) is str:
+            val = val.replace("\\", "\\\\")
+            val = val.replace("\"", "\\\"")
+        return val
+
     def color(self, player):
         if player is self.players[0]:
             return WHITE
@@ -233,6 +247,11 @@ class GameModel(GObject.GObject):
 
     @asyncio.coroutine
     def start_analyzer(self, analyzer_type):
+        # prevent starting new analyzers again and again
+        # when fics lecture reuses the same gamemodel
+        if analyzer_type in self.spectators:
+            return
+
         from pychess.Players.engineNest import init_engine
         analyzer = yield from init_engine(analyzer_type, self)
         if analyzer is None:
@@ -261,7 +280,6 @@ class GameModel(GObject.GObject):
             return
 
         analyzer.resume()
-        analyzer.setOptionInitialBoard(self)
         self.emit("analyzer_resumed", analyzer, analyzer_type)
 
     def pause_analyzer(self, analyzer_type):
@@ -280,14 +298,15 @@ class GameModel(GObject.GObject):
 
     def on_analyze(self, analyzer, analysis):
         if analysis and analysis[0] is not None:
-            pv, score, depth = analysis[0]
-            ply = analyzer.board.ply
+            ply, pv, score, depth, nps = analysis[0]
             if score is not None:
                 if analyzer.mode == ANALYZING:
-                    self.scores[ply] = (pv, score, depth)
-                    self.emit("analysis_changed", ply)
+                    if (ply not in self.scores) or (int(self.scores[ply][2]) <= int(depth)):
+                        self.scores[ply] = (pv, score, depth)
+                        self.emit("analysis_changed", ply)
                 else:
-                    self.spy_scores[ply] = (pv, score, depth)
+                    if (ply not in self.spy_scores) or (int(self.spy_scores[ply][2]) <= int(depth)):
+                        self.spy_scores[ply] = (pv, score, depth)
 
     def setOpening(self, ply=None):
         if ply is None:
@@ -298,7 +317,7 @@ class GameModel(GObject.GObject):
         if ply > 0:
             opening = get_eco(self.getBoardAtPly(ply).board.hash)
         else:
-            opening = ("", "", "")
+            opening = None
         if opening is not None:
             self.tags["ECO"] = opening[0]
             self.tags["Opening"] = opening[1]
@@ -399,7 +418,11 @@ class GameModel(GObject.GObject):
         if self.players and self.status in (WAITING_TO_START, PAUSED, RUNNING):
             if (self.players[0].__type__ == LOCAL and self.players[1].__type__ == REMOTE) or \
                (self.players[1].__type__ == LOCAL and self.players[0].__type__ == REMOTE) or \
-               (self.players[1].__type__ == REMOTE and self.players[0].__type__ == REMOTE and self.examined):
+               (self.players[1].__type__ == REMOTE and self.players[0].__type__ == REMOTE and
+                    self.examined and (
+                    self.connection.offline_lecture or
+                    self.players[0].name == "puzzlebot" or self.players[1].name == "puzzlebot") or
+                    self.players[0].name == "endgamebot" or self.players[1].name == "endgamebot"):
                 return True
         return False
 
@@ -626,7 +649,6 @@ class GameModel(GObject.GObject):
 
             while self.status in (PAUSED, RUNNING, DRAW, WHITEWON, BLACKWON):
                 curPlayer = self.players[self.curColor]
-
                 if self.timed:
                     log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: updating %s's time" % (
                         id(self), str(self.players), str(self.ply), str(curPlayer)))
@@ -656,19 +678,31 @@ class GameModel(GObject.GObject):
                             self.kill(BLACK_ENGINE_DIED)
                     break
                 except InvalidMove as e:
+                    stringio = StringIO()
+                    traceback.print_exc(file=stringio)
+                    error = stringio.getvalue()
+                    log.error(
+                        "GameModel.run: InvalidMove by player=%s error=%s\n%s"
+                        % (curPlayer, error, e))
                     if self.curColor == WHITE:
                         self.end(BLACKWON, WON_ADJUDICATION)
                     else:
                         self.end(WHITEWON, WON_ADJUDICATION)
                     break
+                except PassInterrupt:
+                    log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: PassInterrupt" % (
+                        id(self), str(self.players), self.ply))
+                    continue
                 except TurnInterrupt:
                     log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: TurnInterrupt" % (
                         id(self), str(self.players), self.ply))
                     self.curColor = self.boards[-1].color
                     continue
+                except GameEnded:
+                    log.debug("GameModel.run: got GameEnded exception")
+                    break
 
                 assert isinstance(move, Move), "%s" % repr(move)
-
                 log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: applying move=%s" % (
                     id(self), str(self.players), self.ply, str(move)))
                 self.needsSave = True
@@ -701,6 +735,8 @@ class GameModel(GObject.GObject):
                 self.checkStatus()
                 self.curColor = 1 - self.curColor
 
+            self.checkStatus()
+
         asyncio.async(coro())
 
     def checkStatus(self):
