@@ -1,8 +1,8 @@
 from pychess.Utils.const import EMPTY, PAWN, KNIGHT, BISHOP, ROOK, QUEEN, KING, \
     ATOMICCHESS, BUGHOUSECHESS, CRAZYHOUSECHESS, CAMBODIANCHESS, MAKRUKCHESS, \
     FISCHERRANDOMCHESS, SITTUYINCHESS, WILDCASTLECHESS, WILDCASTLESHUFFLECHESS, \
-    SUICIDECHESS, DROP_VARIANTS, BLACK, WHITE, FAN_PIECES, NULL_MOVE, CAS_FLAGS, \
-    NORMALCHESS, FEN_START, \
+    SUICIDECHESS, GIVEAWAYCHESS, DROP_VARIANTS, BLACK, WHITE, FAN_PIECES, NULL_MOVE, CAS_FLAGS, \
+    HORDECHESS, NORMALCHESS, FEN_START, \
     chrU2Sign, cordDic, reprCord, reprFile, reprSign, reprSignMakruk, reprSignSittuyin, \
     A1, A8, B1, B8, \
     C1, C8, D1, D8, \
@@ -378,7 +378,7 @@ class LBoard(object):
         self.fen_was_applied = True
 
     def isChecked(self):
-        if self.variant == SUICIDECHESS:
+        if self.variant == SUICIDECHESS or self.variant == GIVEAWAYCHESS:
             return False
         elif self.variant == ATOMICCHESS:
             if not self.boards[self.color][KING]:
@@ -389,6 +389,8 @@ class LBoard(object):
             return False
         if self.checked is None:
             kingcord = self.kings[self.color]
+            if self.variant == HORDECHESS and kingcord == -1:
+                return False
             self.checked = isAttacked(self,
                                       kingcord,
                                       1 - self.color,
@@ -396,7 +398,7 @@ class LBoard(object):
         return self.checked
 
     def opIsChecked(self):
-        if self.variant == SUICIDECHESS:
+        if self.variant == SUICIDECHESS or self.variant == GIVEAWAYCHESS:
             return False
         elif self.variant == ATOMICCHESS:
             if not self.boards[1 - self.color][KING]:
@@ -407,6 +409,8 @@ class LBoard(object):
             return False
         if self.opchecked is None:
             kingcord = self.kings[1 - self.color]
+            if self.variant == HORDECHESS and kingcord == -1:
+                return False
             self.opchecked = isAttacked(self,
                                         kingcord,
                                         self.color,
@@ -414,7 +418,7 @@ class LBoard(object):
         return self.opchecked
 
     def willLeaveInCheck(self, move):
-        if self.variant == SUICIDECHESS:
+        if self.variant == SUICIDECHESS or self.variant == GIVEAWAYCHESS:
             return False
         board_clone = self.clone()
         board_clone.applyMove(move)
@@ -995,5 +999,6 @@ class LBoard(object):
         copy.fen_was_applied = self.fen_was_applied
         return copy
 
+
 START_BOARD = LBoard()
 START_BOARD.applyFen(FEN_START)
