@@ -5,9 +5,9 @@ from .ldata import fromToRay, moveArray, directions, fileBits, rankBits,\
     ray45, attack45, ray135, attack135, ray90, attack90, ray00, attack00, FILE, rays
 from pychess.Utils.const import EMPTY, PAWN,\
     QUEEN, KNIGHT, BISHOP, ROOK, KING, WHITE, BLACK,\
-    SITTUYINCHESS, FISCHERRANDOMCHESS, SUICIDECHESS, CAMBODIANCHESS,\
+    SITTUYINCHESS, FISCHERRANDOMCHESS, SUICIDECHESS, GIVEAWAYCHESS, CAMBODIANCHESS,\
     ATOMICCHESS, WILDCASTLECHESS, WILDCASTLESHUFFLECHESS, CRAZYHOUSECHESS, ASEAN_VARIANTS,\
-    BPAWN, sliders,\
+    HORDECHESS, BPAWN, sliders,\
     A8, A6, G6, F6, H1, C3, B2, B3, A3, D6, D8, E3, E1, E8, C7, F2, D1, E6, H3, D3, H2, G7, H6, H7,\
     ASEAN_QUEEN, ASEAN_BBISHOP, ASEAN_WBISHOP, NORMAL_MOVE, QUEEN_CASTLE, KING_CASTLE, ENPASSANT,\
     KNIGHT_PROMOTION, BISHOP_PROMOTION, ROOK_PROMOTION, QUEEN_PROMOTION, KING_PROMOTION, NULL_MOVE,\
@@ -176,7 +176,7 @@ def genPieceMoves(board, piece, tcord):
                     moves.add(newMove(fcord, tcord))
             return moves
 
-    if board.variant == SUICIDECHESS and piece == KING:
+    if (board.variant == SUICIDECHESS or board.variant == GIVEAWAYCHESS) and piece == KING:
         kings = board.boards[board.color][KING]
         if kings:
             kingMoves = moveArray[KING]
@@ -306,6 +306,16 @@ def genAllMoves(board, drops=True):
         for cord in iterBits(movedpawns):
             yield newMove(cord - 16, cord)
 
+        # In horde white pawns on first rank may move two squares also
+        if board.variant == HORDECHESS:
+            firstrow = pawns & rankBits[0]  # Get first row pawns
+            movedpawns = (firstrow >>
+                          8) & notblocker  # Move two steps forward, while
+            movedpawns = (movedpawns >>
+                          8) & notblocker  # ensuring middle cord is clear
+            for cord in iterBits(movedpawns):
+                yield newMove(cord - 16, cord)
+
         # Capture left
 
         capLeftPawns = pawns & ~fileBits[0]
@@ -313,7 +323,7 @@ def genAllMoves(board, drops=True):
         for cord in iterBits(capLeftPawns):
             if cord in promotion_zone:
                 for p in PROMOTIONS:
-                    if board.variant == SUICIDECHESS or p != KING_PROMOTION:
+                    if (board.variant == SUICIDECHESS or board.variant == GIVEAWAYCHESS) or p != KING_PROMOTION:
                         yield newMove(cord - 7, cord, p)
             elif cord == enpassant:
                 yield newMove(cord - 7, cord, ENPASSANT)
@@ -344,7 +354,7 @@ def genAllMoves(board, drops=True):
         for cord in iterBits(movedpawns):
             if cord in promotion_zone:
                 for p in PROMOTIONS:
-                    if board.variant == SUICIDECHESS or p != KING_PROMOTION:
+                    if (board.variant == SUICIDECHESS or board.variant == GIVEAWAYCHESS) or p != KING_PROMOTION:
                         yield newMove(cord + 8, cord, p)
             else:
                 yield newMove(cord + 8, cord)
