@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
-
+import asyncio
 import datetime
 from gi.repository import GLib, GObject
 
 from pychess.System.Log import log
-from pychess.Utils.IconLoader import load_icon
+from pychess.Utils.IconLoader import load_icon, get_pixbuf
 from pychess.Utils.const import ADJOURNED, WHITE, BLACK, UNSUPPORTED
 
 from pychess.ic import RATING_TYPES, IC_STATUS_PLAYING, IC_STATUS_OFFLINE, IC_STATUS_UNKNOWN, \
@@ -297,15 +297,15 @@ class FICSPlayer(GObject.GObject):
     def getIconByRating(cls, rating, size=16):
         assert isinstance(rating, int), "rating not an int: %s" % str(rating)
         if rating >= 1900:
-            return load_icon(size, "weather-storm")
+            return get_pixbuf("glade/16x16/weather-storm.png")
         elif rating >= 1600:
-            return load_icon(size, "weather-showers")
+            return get_pixbuf("glade/16x16/weather-showers.png")
         elif rating >= 1300:
-            return load_icon(size, "weather-overcast")
+            return get_pixbuf("glade/16x16/weather-overcast.png")
         elif rating >= 1000:
-            return load_icon(size, "weather-few-clouds")
+            return get_pixbuf("glade/16x16/weather-few-clouds.png")
         else:
-            return load_icon(size, "weather-clear")
+            return get_pixbuf("glade/16x16/weather-clear.png")
 
     def getIcon(self, size=16, gametype=None):
         assert isinstance(size, int), "size not an int: %s" % str(size)
@@ -863,6 +863,9 @@ class FICSGame(FICSMatch):
         self.private = private
         self.relation = relation
 
+        # move(style12) message queue feeded by BoardManager, consumed by balck and white ICPlayer
+        self.move_queue = asyncio.Queue()
+
     def __hash__(self):
         return hash(":".join((self.wplayer.name[0:11].lower(
         ), self.bplayer.name[0:11].lower(), str(self.gameno))))
@@ -1207,9 +1210,20 @@ class FICSGames(GObject.GObject):
 
     def game_ended(self, game):
         if game in self:
-            game = self[game]
-            del self[game]
-            self.emit("FICSGameEnded", game)
+            if not game.move_queue.empty():
+                def coro(game):
+                    # we have to give a chance to ICPlayer
+                    # to process the latest move(style12) message
+                    # and remove game from game list panel also
+                    yield from asyncio.sleep(0)
+                    game = self[game]
+                    self.emit("FICSGameEnded", game)
+                    del self[game]
+                asyncio.async(coro(game))
+            else:
+                game = self[game]
+                self.emit("FICSGameEnded", game)
+                del self[game]
 
     def onAdjournmentsList(self, adm, adjournments):
         for game in self.adjourned_games.values():
