@@ -1,4 +1,3 @@
-
 import re
 import asyncio
 
@@ -17,10 +16,9 @@ from pychess.Utils.const import WHITEWON, WON_RESIGN, WON_DISCONNECTION, WON_CAL
     ABORTED, ABORTED_AGREEMENT, ABORTED_DISCONNECTION, ABORTED_EARLY, ABORTED_SERVER_SHUTDOWN, \
     ABORTED_ADJUDICATION, ABORTED_COURTESY, UNKNOWN_STATE, BLACK, WHITE, reprFile, \
     FISCHERRANDOMCHESS, CRAZYHOUSECHESS, WILDCASTLECHESS, WILDCASTLESHUFFLECHESS, ATOMICCHESS, \
-    LOSERSCHESS, SUICIDECHESS, reprResult
+    LOSERSCHESS, SUICIDECHESS, GIVEAWAYCHESS, reprResult
 
-from pychess.ic import IC_POS_INITIAL, IC_POS_ISOLATED, IC_POS_OP_TO_MOVE, IC_POS_ME_TO_MOVE, \
-    IC_POS_OBSERVING, IC_POS_OBSERVING_EXAMINATION, IC_POS_EXAMINATING, GAME_TYPES, IC_STATUS_PLAYING, \
+from pychess.ic import IC_POS_OBSERVING_EXAMINATION, IC_POS_EXAMINATING, GAME_TYPES, IC_STATUS_PLAYING, \
     BLKCMD_SEEK, BLKCMD_OBSERVE, BLKCMD_MATCH, TYPE_WILD, BLKCMD_SMOVES, BLKCMD_UNOBSERVE, BLKCMD_MOVES, \
     BLKCMD_FLAG, parseRating
 
@@ -66,14 +64,6 @@ sr = re.compile("<sr> ([\d ]+)")
 fileToEpcord = (("a3", "b3", "c3", "d3", "e3", "f3", "g3", "h3"),
                 ("a6", "b6", "c6", "d6", "e6", "f6", "g6", "h6"))
 
-relations = {"-4": IC_POS_INITIAL,
-             "-3": IC_POS_ISOLATED,
-             "-2": IC_POS_OBSERVING_EXAMINATION,
-             "2": IC_POS_EXAMINATING,
-             "-1": IC_POS_OP_TO_MOVE,
-             "1": IC_POS_ME_TO_MOVE,
-             "0": IC_POS_OBSERVING}
-
 
 def parse_reason(result, reason, wname=None):
     """
@@ -188,9 +178,12 @@ class BoardManager(GObject.GObject):
         'playGameCreated': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
         'obsGameCreated': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
         'exGameCreated': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
-        'exGameBackward': (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
+        'exGameReset': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
+        'gameUndoing': (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
         'archiveGamePreview': (GObject.SignalFlags.RUN_FIRST, None,
                                (object, )),
+        'boardSetup': (GObject.SignalFlags.RUN_FIRST, None,
+                       (int, str, str, str)),
         'timesUpdate': (GObject.SignalFlags.RUN_FIRST, None,
                         (int, int, int,)),
         'obsGameEnded': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
@@ -311,6 +304,10 @@ class BoardManager(GObject.GObject):
         self.connection.expect_line(self.made_unexamined,
                                     "You are no longer examining game (\d+)\.")
 
+        self.connection.expect_n_lines(
+            self.onExamineGameCreated, "Starting a game in examine \(scratch\) mode\.",
+            '', "<12> (.+)")
+
         self.queuedEmits = {}
         self.gamemodelStartedEvents = {}
         self.theGameImPlaying = None
@@ -343,9 +340,12 @@ class BoardManager(GObject.GObject):
 
         curcol = fields[8] == "B" and BLACK or WHITE
         gameno = int(fields[15])
-        relation = relations[fields[18]]
-        ply = int(fields[25]) * 2 - (curcol == WHITE and 2 or 1)
+        relation = int(fields[18])
         lastmove = fields[28] != "none" and fields[28] or None
+        if lastmove is None:
+            ply = 0
+        else:
+            ply = int(fields[25]) * 2 - (curcol == WHITE and 2 or 1)
         wname = fields[16]
         bname = fields[17]
         wms = int(fields[23])
@@ -409,6 +409,7 @@ class BoardManager(GObject.GObject):
 
     def onStyle12(self, match):
         style12 = match.groups()[0]
+        log.debug("onStyle12: %s" % style12)
         gameno = int(style12.split()[15])
         if gameno in self.queuedStyle12s:
             self.queuedStyle12s[gameno].append(style12)
@@ -437,38 +438,52 @@ class BoardManager(GObject.GObject):
             wplayer = self.connection.players.get(wname)
             bplayer = self.connection.players.get(bname)
 
-            # examine from console or got mexamine in observed game
             if self.connection.examined_game is None:
-                no_smoves = True
-                game = FICSGame(wplayer,
-                                bplayer,
-                                gameno=int(gameno),
-                                game_type=GAME_TYPES["examined"],
-                                minutes=0,
-                                inc=0,
-                                board=FICSBoard(0,
-                                                0,
-                                                pgn=pgn),
-                                relation=relation)
-                self.connection.examined_game = game
-            else:
                 # examine an archived game from GUI
-                no_smoves = False
-                game = self.connection.examined_game
-                game.gameno = int(gameno)
-                game.relation = relation
-                # game.game_type = GAME_TYPES["examined"]
-            game = self.connection.games.get(game)
+                if self.connection.archived_examine is not None:
+                    no_smoves = False
+                    game = self.connection.archived_examine
+                    game.gameno = int(gameno)
+                    game.relation = relation
+                    # game.game_type = GAME_TYPES["examined"]
+                    log.debug("Start examine an existing game by %s" % style12,
+                              extra={"task": (self.connection.username, "BM.onStyle12")})
+                else:
+                    # examine from console or got mexamine in observed game
+                    no_smoves = True
+                    game = FICSGame(wplayer,
+                                    bplayer,
+                                    gameno=int(gameno),
+                                    game_type=GAME_TYPES["examined"],
+                                    minutes=0,
+                                    inc=0,
+                                    board=FICSBoard(0,
+                                                    0,
+                                                    pgn=pgn),
+                                    relation=relation)
+                    log.debug("Start new examine game by %s" % style12,
+                              extra={"task": (self.connection.username, "BM.onStyle12")})
 
-            # For examined games we have to create move queues here
-            # because game will be started later by generalStart()
-            if not hasattr(game, "queue"):
-                game.queue = asyncio.Queue()
+                game = self.connection.games.get(game)
+                self.connection.examined_game = game
 
-            # don't start new game in puzzlebot/endgamebot when they just reuse gameno
-            if game.relation == IC_POS_OBSERVING_EXAMINATION or \
-                    (game.board is not None and game.board.pgn == pgn):
-                game.queue.put_nowait((gameno, ply, curcol, lastmove, fen, wname, bname, wms, bms))
+                # don't start another new game when someone (some human examiner or lecturebot/endgamebot)
+                # changes our relation in an already started game from IC_POS_OBSERVING_EXAMINATION to
+                # IC_POS_EXAMINATING
+                if game.relation == IC_POS_OBSERVING_EXAMINATION:
+                    game.relation = relation  # IC_POS_EXAMINATING
+                    # print("IC_POS_OBSERVING_EXAMINATION --> IC_POS_EXAMINATING")
+                    # before this change server sent an unobserve to us
+                    # and it removed gameno from started events dict
+                    # we have to put it back...
+                    self.gamemodelStartedEvents[game.gameno] = asyncio.Event()
+                    return
+
+            else:
+                # don't start new game in puzzlebot/endgamebot when they just reuse gameno
+                log.debug("emit('boardSetup') with %s %s %s %s" % (gameno, fen, wname, bname),
+                          extra={"task": (self.connection.username, "BM.onStyle12")})
+                self.emit("boardSetup", gameno, fen, wname, bname)
                 return
 
             game.relation = relation
@@ -478,9 +493,13 @@ class BoardManager(GObject.GObject):
             # start a new game now or after smoves
             self.gamemodelStartedEvents[game.gameno] = asyncio.Event()
             if no_smoves:
+                log.debug("emit('exGameCreated')",
+                          extra={"task": (self.connection.username, "BM.onStyle12")})
                 self.emit("exGameCreated", game)
                 self.gamemodelStartedEvents[game.gameno].wait()
             else:
+                log.debug("send 'smoves' command",
+                          extra={"task": (self.connection.username, "BM.onStyle12")})
                 if isinstance(game, FICSHistoryGame):
                     self.connection.client.run_command("smoves %s %s" % (
                         self.connection.history_owner, game.history_no))
@@ -494,14 +513,66 @@ class BoardManager(GObject.GObject):
         else:
             if gameno in self.connection.games.games_by_gameno:
                 game = self.connection.games.get_game_by_gameno(gameno)
-                if not hasattr(game, "queue"):
-                    game.queue = asyncio.Queue()
-                game.queue.put_nowait((gameno, ply, curcol, lastmove, fen, wname, bname, wms, bms))
+                if wms < 0 or bms < 0:
+                    # fics resend latest style12 line again when one player lost on time
+                    return
+                if lastmove is None:
+                    log.debug("emit('boardSetup') with %s %s %s %s" % (gameno, fen, wname, bname),
+                              extra={"task": (self.connection.username, "BM.onStyle12")})
+                    self.emit("boardSetup", gameno, fen, wname, bname)
+                else:
+                    log.debug("put move %s into game.move_queue" % lastmove,
+                              extra={"task": (self.connection.username, "BM.onStyle12")})
+                    game.move_queue.put_nowait((gameno, ply, curcol, lastmove, fen, wname, bname, wms, bms))
             else:
                 # In some cases (like lost on time) the last move is resent by FICS
                 # but game was already removed from self.connection.games
                 log.debug("Got %s but %s not in connection.games" % (style12, gameno))
 
+    def onExamineGameCreated(self, matchlist):
+        style12 = matchlist[-1].groups()[0]
+        gameno = int(style12.split()[15])
+
+        castleSigns = self.generateCastleSigns(style12, GAME_TYPES["examined"])
+        self.castleSigns[gameno] = castleSigns
+        gameno, relation, curcol, ply, wname, bname, wms, bms, gain, lastmove, fen = \
+            self.parseStyle12(style12, castleSigns)
+
+        pgnHead = [
+            ("Event", "FICS examined game"), ("Site", "freechess.org"),
+            ("White", wname), ("Black", bname), ("Result", "*"),
+            ("SetUp", "1"), ("FEN", fen)
+        ]
+        pgn = "\n".join(['[%s "%s"]' % line for line in pgnHead]) + "\n*\n"
+        wplayer = self.connection.players.get(wname)
+        bplayer = self.connection.players.get(bname)
+
+        game = FICSGame(wplayer,
+                        bplayer,
+                        gameno=int(gameno),
+                        game_type=GAME_TYPES["examined"],
+                        minutes=0,
+                        inc=0,
+                        board=FICSBoard(0,
+                                        0,
+                                        pgn=pgn),
+                        relation=relation)
+        log.debug("Starting a game in examine (scratch) mode.",
+                  extra={"task": (self.connection.username, "BM.onExamineGameCreated")})
+        self.connection.examined_game = game
+        game = self.connection.games.get(game)
+
+        game.relation = relation
+        game.board = FICSBoard(0, 0, pgn=pgn)
+        self.gamesImObserving[game] = wms, bms
+
+        # start a new game now
+        self.gamemodelStartedEvents[game.gameno] = asyncio.Event()
+        log.debug("emit('exGameCreated')",
+                  extra={"task": (self.connection.username, "BM.onExamineGameCreated")})
+        self.emit("exGameCreated", game)
+        self.gamemodelStartedEvents[game.gameno].wait()
+
     def onGameModelStarted(self, gameno):
         self.gamemodelStartedEvents[gameno].set()
 
@@ -865,6 +936,8 @@ class BoardManager(GObject.GObject):
             pgnHead += [("Variant", "Losers")]
         elif game_type.variant_type == SUICIDECHESS:
             pgnHead += [("Variant", "Suicide")]
+        elif game_type.variant_type == GIVEAWAYCHESS:
+            pgnHead += [("Variant", "Giveaway")]
         pgn = "\n".join(['[%s "%s"]' % line for line in pgnHead]) + "\n"
 
         moves = sorted(moves.items())
@@ -934,19 +1007,11 @@ class BoardManager(GObject.GObject):
         if wplayer.game is not None:
             game.rated = wplayer.game.rated
         game = self.connection.games.get(game, emit=False)
-
-        def coro():
-            # sometimes the latest move of the game comes after on_game_remove
-            # we have wait a little here before removing game from connection.games
-            # to let pychess a chance to processed it
-            yield from asyncio.wait(0.1)
-            self.connection.games.game_ended(game)
-            # Do this last to give anybody connected to the game's signals a chance
-            # to disconnect from them first
-            wplayer.game = None
-            bplayer.game = None
-
-        asyncio.async(coro())
+        self.connection.games.game_ended(game)
+        # Do this last to give anybody connected to the game's signals a chance
+        # to disconnect from them first
+        wplayer.game = None
+        bplayer.game = None
 
     def onObserveGameCreated(self, matchlist):
         log.debug("'%s'" % (matchlist[1].string),
@@ -993,9 +1058,22 @@ class BoardManager(GObject.GObject):
                                             pgn=pgn),
                             relation=relation)
             game = self.connection.games.get(game)
+
+            # when puzzlebot reuses same gameno for starting next puzzle
+            # sometimes no unexamine sent by server, so we have to set None to
+            # self.connection.examined_game to guide self.onStyle12() a bit...
+            if self.connection.examined_game is not None and \
+                    self.connection.examined_game.gameno == gameno:
+                log.debug("BM.onObserveGameCreated: exGameReset emitted; self.connection.examined_game = %s" % gameno)
+                self.emit("exGameReset", self.connection.examined_game)
+                self.connection.examined_game = None
+
+            game.relation = relation  # IC_POS_OBSERVING_EXAMINATION
             self.gamesImObserving[game] = wms, bms
 
             self.gamemodelStartedEvents[game.gameno] = asyncio.Event()
+            # puzzlebot sometimes creates next puzzle with same wplayer,bplayer,gameno
+            game.move_queue = asyncio.Queue()
             self.emit("obsGameCreated", game)
             self.gamemodelStartedEvents[game.gameno].wait()
         else:
@@ -1038,7 +1116,6 @@ class BoardManager(GObject.GObject):
             return
         if game.gameno not in self.queuedEmits:
             return
-
         self.emit("obsGameCreated", game)
         try:
             self.gamemodelStartedEvents[game.gameno].wait()
@@ -1169,7 +1246,9 @@ class BoardManager(GObject.GObject):
         self.emit("madeExamined", gameno)
 
     def made_unexamined(self, match):
-        """You are no longer examine game"""
+        """ You are no longer examine game """
+        log.debug("BM.made_unexamined(): exGameReset emitted")
+        self.emit("exGameReset", self.connection.examined_game)
         self.connection.examined_game = None
         gameno, = match.groups()
         gameno = int(gameno)
