@@ -5,7 +5,7 @@ from gi.repository import Gdk, Gtk, GObject, Pango, PangoCairo
 
 from pychess.System import conf, uistuff
 from pychess.Utils import prettyPrintScore
-from pychess.Utils.const import HINT, OPENING, SPY, BLACK, NULL_MOVE, ENDGAME, DRAW, WHITEWON, WHITE
+from pychess.Utils.const import HINT, OPENING, SPY, BLACK, NULL_MOVE, ENDGAME, DRAW, WHITEWON, WHITE, NORMALCHESS
 from pychess.Utils.book import getOpenings
 from pychess.Utils.eco import get_eco
 from pychess.Utils.logic import legalMoveCount
@@ -98,9 +98,7 @@ class OpeningAdvisor(Advisor):
 
     def shownChanged(self, boardview, shown):
         m = boardview.model
-        if m is None:
-            return
-        if m.isPlayingICSGame():
+        if m is None or m.variant.variant != NORMALCHESS or m.isPlayingICSGame():
             return
 
         b = m.getBoardAtPly(shown, boardview.shown_variation_idx)
@@ -185,16 +183,17 @@ class EngineAdvisor(Advisor):
             return
         if m.isPlayingICSGame():
             return
-        if not self.active:
-            return
 
         self.engine.setBoard(boardview.model.getBoardAtPly(
-            shown, boardview.shown_variation_idx))
-        self._create_new_expected_lines()
+            shown, boardview.shown_variation_idx), search=self.active)
+
+        if self.active:
+            self._create_new_expected_lines()
 
     def on_ready_for_options(self, engine):
-        engineMax = self.engine.maxAnalysisLines()
-        self.linesExpected = min(conf.get("multipv", 1), engineMax)
+        engine_max = self.engine.maxAnalysisLines()
+        engine_value = self.engine.getAnalysisLines()
+        self.linesExpected = engine_value if engine_value <= engine_max else engine_max
 
         m = self.boardview.model
         if m.isPlayingICSGame():
@@ -203,10 +202,9 @@ class EngineAdvisor(Advisor):
         parent = self._create_new_expected_lines()
 
         # set pvlines, but set it 0 if engine max is only 1
-        self.store.set_value(parent, 2, 0
-                             if engineMax == 1 else self.linesExpected)
+        self.store.set_value(parent, 2, 0 if engine_max == 1 else self.linesExpected)
         # set it editable
-        self.store.set_value(parent, 3, engineMax > 1)
+        self.store.set_value(parent, 3, engine_max > 1)
         # set start/stop cb visible
         self.store.set_value(parent, 6, True)
         self.active = True
@@ -230,20 +228,17 @@ class EngineAdvisor(Advisor):
                 self.store[self.path + (i, )] = self.textOnlyRow("")
                 continue
 
+            ply, movstrs, score, depth, nps = line
             board0 = self.engine.board
             board = board0.clone()
-
-            movstrs, score, depth = line
             try:
                 pv = listToMoves(board, movstrs, validate=True)
             except ParsingError as e:
                 # ParsingErrors may happen when parsing "old" lines from
                 # analyzing engines, which haven't yet noticed their new tasks
-                log.debug("__parseLine: Ignored (%s) from analyzer: ParsingError%s" %
+                log.debug("EngineAdvisor.on_analyze(): Ignored (%s) from analyzer: ParsingError%s" %
                           (' '.join(movstrs), e))
                 return
-            except:
-                return
 
             move = None
             if pv:
@@ -278,17 +273,13 @@ class EngineAdvisor(Advisor):
         if not tb:
             self.active = True
             self.boardview.model.resume_analyzer(self.mode)
-            self.shownChanged(self.boardview, self.boardview.shown)
         else:
             self.active = False
             self.boardview.model.pause_analyzer(self.mode)
 
     def multipv_edited(self, value):
-        if value > self.engine.maxAnalysisLines():
-            return
-
+        value = self.engine.requestMultiPV(value)
         if value != self.linesExpected:
-            self.engine.requestMultiPV(value)
             parent = self.store.get_iter(self.path)
             if value > self.linesExpected:
                 while self.linesExpected < value:
@@ -301,8 +292,12 @@ class EngineAdvisor(Advisor):
                     if child is not None:
                         self.store.remove(child)
                     self.linesExpected -= 1
+        return value
 
     def row_activated(self, iter, model):
+        if not self.active:
+            return
+
         if self.mode == HINT and self.store.get_path(iter) != Gtk.TreePath(self.path):
             moves = self.store[iter][0][2]
             if moves is not None:
@@ -363,9 +358,7 @@ class EndgameAdvisor(Advisor):
 
     def shownChanged(self, boardview, shown):
         m = boardview.model
-        if m is None:
-            return
-        if m.isPlayingICSGame():
+        if m is None or m.variant.variant != NORMALCHESS or m.isPlayingICSGame():
             return
 
         self.parent = self.empty_parent()
@@ -464,7 +457,7 @@ class Sidepanel(object):
 
         # ## multipv (number of analysis lines)
         self.multipvRenderer = Gtk.CellRendererSpin()
-        adjustment = Gtk.Adjustment(value=conf.get("multipv", 1),
+        adjustment = Gtk.Adjustment(value=1,
                                     lower=1,
                                     upper=9,
                                     step_incr=1)
@@ -485,8 +478,7 @@ class Sidepanel(object):
 
         def multipv_edited(renderer, path, text):
             iter = self.store.get_iter(path)
-            self.store.set_value(iter, 2, int(text))
-            self.advisors[int(path[0])].multipv_edited(int(text))
+            self.store.set_value(iter, 2, self.advisors[int(path[0])].multipv_edited(1 if text == "" else int(text)))
 
         self.multipv_cid = self.multipvRenderer.connect('edited', multipv_edited)
 
@@ -502,9 +494,9 @@ class Sidepanel(object):
                 cell.set_property('visible', True)
 
             if store[iter][5]:
-                cell.set_property("stock-id", "gtk-add")
+                cell.set_property("stock-id", "gtk-media-play")
             else:
-                cell.set_property("stock-id", "gtk-remove")
+                cell.set_property("stock-id", "gtk-stop")
 
         c4.set_cell_data_func(self.toggleRenderer, cb_visible)
 
@@ -534,23 +526,21 @@ class Sidepanel(object):
         self.advisors = []
         self.conf_conids = []
 
-        if conf.get("opening_check", 0):
+        if conf.get("opening_check", False):
             advisor = OpeningAdvisor(self.store, self.tv, self.boardview)
             self.advisors.append(advisor)
-        if conf.get("endgame_check", 0):
+        if conf.get("endgame_check", False):
             advisor = EndgameAdvisor(self.store, self.tv, self.boardview)
             self.advisors.append(advisor)
 
         self.model_cids = [
             gmwidg.gamemodel.connect("analyzer_added", self.on_analyzer_added),
             gmwidg.gamemodel.connect("analyzer_removed", self.on_analyzer_removed),
-            gmwidg.gamemodel.connect("analyzer_paused", self.on_analyzer_paused),
-            gmwidg.gamemodel.connect("analyzer_resumed", self.on_analyzer_resumed),
             gmwidg.gamemodel.connect_after("game_terminated", self.on_game_terminated),
         ]
 
         def on_opening_check(none):
-            if conf.get("opening_check", 0) and self.boardview is not None:
+            if conf.get("opening_check", False) and self.boardview is not None:
                 advisor = OpeningAdvisor(self.store, self.tv, self.boardview)
                 self.advisors.append(advisor)
                 advisor.shownChanged(self.boardview, self.boardview.shown)
@@ -575,7 +565,7 @@ class Sidepanel(object):
         self.conf_conids.append(conf.notify_add("opening_file_entry", on_opening_file_entry_changed))
 
         def on_endgame_check(none):
-            if conf.get("endgame_check", 0):
+            if conf.get("endgame_check", False):
                 advisor = EndgameAdvisor(self.store, self.tv, self.boardview)
                 self.advisors.append(advisor)
                 advisor.shownChanged(self.boardview, self.boardview.shown)
@@ -621,20 +611,6 @@ class Sidepanel(object):
                 self.store.remove(parent)
                 self.advisors.remove(advisor)
 
-    def on_analyzer_paused(self, gamemodel, analyzer, analyzer_type):
-        for advisor in self.advisors:
-            if advisor.mode == analyzer_type:
-                advisor.active = False
-                self.store[advisor.path][5] = True
-                advisor.empty_parent()
-
-    def on_analyzer_resumed(self, gamemodel, analyzer, analyzer_type):
-        for advisor in self.advisors:
-            if advisor.mode == analyzer_type:
-                self.store[advisor.path][5] = False
-                advisor.active = True
-                advisor.shownChanged(self.boardview, self.boardview.shown)
-
     def shownChanged(self, boardview, shown):
         if boardview.model is None:
             return
