@@ -3,6 +3,7 @@ import os.path
 import gettext
 import locale
 
+from math import pi
 from operator import attrgetter
 from itertools import groupby
 from io import StringIO
@@ -47,8 +48,6 @@ big_people = load_icon(48, "stock_people", "system-users")
 iwheels = load_icon(24, "gtk-execute", "system-run")
 ipeople = load_icon(24, "stock_people", "system-users")
 inotebook = load_icon(24, "stock_notebook", "computer")
-speople = load_icon(16, "stock_people", "system-users")
-snotebook = load_icon(16, "stock_notebook", "computer")
 
 weather_icons = ("clear", "clear-night", "few-clouds", "few-clouds-night",
                  "fog", "overcast", "severe-alert", "showers-scattered",
@@ -57,40 +56,43 @@ skillToIcon = {}
 # Used by TaskerManager. Put here to help synchronization
 skillToIconLarge = {}
 for i, icon in enumerate(weather_icons, start=1):
-    skillToIcon[2 * i - 1] = load_icon(16, "weather-%s" % icon)
-    skillToIcon[2 * i] = load_icon(16, "weather-%s" % icon)
-    skillToIconLarge[2 * i - 1] = load_icon(48, "weather-%s" % icon)
-    skillToIconLarge[2 * i] = load_icon(48, "weather-%s" % icon)
+    skillToIcon[2 * i - 1] = get_pixbuf("glade/16x16/weather-%s.png" % icon)
+    skillToIcon[2 * i] = get_pixbuf("glade/16x16/weather-%s.png" % icon)
+    skillToIconLarge[2 * i - 1] = get_pixbuf("glade/48x48/weather-%s.png" % icon)
+    skillToIconLarge[2 * i] = get_pixbuf("glade/48x48/weather-%s.png" % icon)
 
 playerItems = []
-smallPlayerItems = []
 analyzerItems = []
+allEngineItems = []
 
 
 def createPlayerUIGlobals(discoverer):
     global playerItems
-    global smallPlayerItems
     global analyzerItems
+    global allEngineItems
 
     playerItems = []
-    smallPlayerItems = []
     analyzerItems = []
+    allEngineItems = []
 
     for variantClass in variants.values():
         playerItems += [[(ipeople, _("Human Being"))]]
-        smallPlayerItems += [[(speople, _("Human Being"))]]
     for engine in discoverer.getEngines():
         name = engine["name"]
         c = discoverer.getCountry(engine)
         path = addDataPrefix("flags/%s.png" % c)
+
         if c and os.path.isfile(path):
             flag_icon = get_pixbuf(path)
         else:
             path = addDataPrefix("flags/unknown.png")
             flag_icon = get_pixbuf(path)
+
+        allEngineItems.append((flag_icon, name))
+
         for variant in discoverer.getEngineVariants(engine):
             playerItems[variant] += [(flag_icon, name)]
-            smallPlayerItems[variant] += [(snotebook, name)]
+
         if discoverer.is_analyzer(engine):
             analyzerItems.append((flag_icon, name))
 
@@ -119,9 +121,27 @@ class _GameInitializationMode(object):
 
     @classmethod
     def _init(cls):
+        cls.white = get_pixbuf("glade/white.png")
+        cls.black = get_pixbuf("glade/black.png")
+
         cls.widgets = uistuff.GladeWidgets("newInOut.glade")
         cls.widgets["newgamedialog"].set_transient_for(mainwindow())
 
+        def on_exchange_players(widget, button_event):
+            white = cls.widgets["whitePlayerCombobox"].get_active()
+            black = cls.widgets["blackPlayerCombobox"].get_active()
+            whiteLevel = cls.widgets["skillSlider1"].get_value()
+            blackLevel = cls.widgets["skillSlider2"].get_value()
+            cls.widgets["whitePlayerCombobox"].set_active(black)
+            cls.widgets["blackPlayerCombobox"].set_active(white)
+            cls.widgets["skillSlider1"].set_value(blackLevel)
+            cls.widgets["skillSlider2"].set_value(whiteLevel)
+
+        cls.widgets["whitePlayerButton"].set_image(Gtk.Image.new_from_pixbuf(cls.white))
+        cls.widgets["whitePlayerButton"].connect("button-press-event", on_exchange_players)
+        cls.widgets["blackPlayerButton"].set_image(Gtk.Image.new_from_pixbuf(cls.black))
+        cls.widgets["blackPlayerButton"].connect("button-press-event", on_exchange_players)
+
         uistuff.createCombo(cls.widgets["whitePlayerCombobox"],
                             name="whitePlayerCombobox")
         uistuff.createCombo(cls.widgets["blackPlayerCombobox"],
@@ -424,26 +444,38 @@ class _GameInitializationMode(object):
                 moves = cls.ngclassical_moves.get_value_as_int()
 
             # Find players
-            player0 = cls.widgets["whitePlayerCombobox"].get_active()
-            player0 = playerItems[0].index(playerItems[variant_index][player0])
+            player0combo = cls.widgets["whitePlayerCombobox"]
+            player0 = player0combo.get_active()
+
+            tree_iter = player0combo.get_active_iter()
+            if tree_iter is not None:
+                model = player0combo.get_model()
+                name0 = model[tree_iter][1]
+
             diffi0 = int(cls.widgets["skillSlider1"].get_value())
-            player1 = cls.widgets["blackPlayerCombobox"].get_active()
-            player1 = playerItems[0].index(playerItems[variant_index][player1])
+
+            player1combo = cls.widgets["blackPlayerCombobox"]
+            player1 = player1combo.get_active()
+
+            tree_iter = player1combo.get_active_iter()
+            if tree_iter is not None:
+                model = player1combo.get_model()
+                name1 = model[tree_iter][1]
+
             diffi1 = int(cls.widgets["skillSlider2"].get_value())
 
             # Prepare players
             playertups = []
-            for i, playerno, diffi, color in ((0, player0, diffi0, WHITE),
-                                              (1, player1, diffi1, BLACK)):
+            for i, playerno, name, diffi, color in ((0, player0, name0, diffi0, WHITE),
+                                                    (1, player1, name1, diffi1, BLACK)):
                 if playerno > 0:
-                    engine = discoverer.getEngineN(playerno - 1)
-                    name = discoverer.getName(engine)
+                    engine = discoverer.getEngineByName(name)
                     playertups.append((ARTIFICIAL, discoverer.initPlayerEngine,
                                        [engine, color, diffi, variant, secs,
                                         incr, moves], name))
                 else:
                     if not playertups or playertups[0][0] != LOCAL:
-                        name = conf.get("firstName", _("You"))
+                        name = conf.get("firstName", conf.username)
                     else:
                         name = conf.get("secondName", _("Guest"))
                     playertups.append((LOCAL, Human, (color, name), name))
@@ -526,11 +558,12 @@ class SetupPositionExtension(_GameInitializationMode):
 
         cls.castl = set()
 
-        cls.white = Gtk.Image.new_from_pixbuf(get_pixbuf("glade/white.png"))
-        cls.black = Gtk.Image.new_from_pixbuf(get_pixbuf("glade/black.png"))
+        cls.white = Gtk.Image.new_from_pixbuf(cls.white)
+        cls.black = Gtk.Image.new_from_pixbuf(cls.black)
         cls.widgets["side_button"].set_image(cls.white)
 
         cls.widgets["side_button"].connect("toggled", cls.side_button_toggled)
+        cls.widgets["rotate_button"].connect("button-press-event", cls.rotate_button_pressed)
         cls.widgets["moveno_spin"].connect("value-changed",
                                            cls.moveno_spin_changed)
         cls.widgets["fifty_spin"].connect("value-changed",
@@ -561,6 +594,14 @@ class SetupPositionExtension(_GameInitializationMode):
             button.set_image(cls.white)
         cls.fen_changed()
 
+    @classmethod
+    def rotate_button_pressed(self, widget, button_event):
+        view = self.board_control.view
+        if view.rotation:
+            view.rotation = 0
+        else:
+            view.rotation = pi
+
     @classmethod
     def fen_changed(cls):
         cls.widgets["fen_entry"].set_text(cls.get_fen())
@@ -618,7 +659,6 @@ class SetupPositionExtension(_GameInitializationMode):
         cls.widgets["moveno_spin"].set_value(lboard.plyCount // 2 + 1)
         ep = lboard.enpassant
         cls.widgets["ep_combo"].set_active(0 if ep is None else FILE(ep) + 1)
-        cls.castl = set()
         cls.widgets["woo"].set_active(lboard.castling & W_OO)
         cls.widgets["wooo"].set_active(lboard.castling & W_OOO)
         cls.widgets["boo"].set_active(lboard.castling & B_OO)
@@ -704,8 +744,10 @@ class EnterNotationExtension(_GameInitializationMode):
         else:
             path = gettext.find("pychess", localedir=addDataPrefix("lang"))
         if path:
-            loc = locale.getdefaultlocale()[0][-2:].lower()
-            flags.append(addDataPrefix("flags/%s.png" % loc))
+            default_locale = locale.getdefaultlocale()
+            if default_locale[0] is not None:
+                loc = locale.getdefaultlocale()[0][-2:].lower()
+                flags.append(addDataPrefix("flags/%s.png" % loc))
 
         flags.append(addDataPrefix("flags/us.png"))
 
@@ -787,6 +829,19 @@ class EnterNotationExtension(_GameInitializationMode):
             elif parts_no > 0 and text.split()[0].count("/") == 7:
                 loadType = fen
             else:
+                # patch default human player names on demand...
+                player0combo = cls.widgets["whitePlayerCombobox"]
+                player0 = player0combo.get_active()
+                if player0 == 0 and '[White "' not in text:
+                    name = '[White "%s"]' % conf.get("firstName", conf.username)
+                    text = "%s\n%s" % (name, text)
+
+                player1combo = cls.widgets["blackPlayerCombobox"]
+                player1 = player1combo.get_active()
+                if player1 == 0 and '[Black "' not in text:
+                    name = '[Black "%s"]' % conf.get("secondName", _("Guest"))
+                    text = "%s\n%s" % (name, text)
+
                 loadType = pgn
 
             return text, loadType
