@@ -45,8 +45,12 @@ def initialize(widgets):
     GeneralTab(widgets)
     HintTab(widgets)
     SoundTab(widgets)
-    PanelTab(widgets)
+
+    # All side panels can be show/hide from View menu now, so no need to do the same from preferences
+    # We can re enable this after implementing install/uninstall functionality in the future...
+    # PanelTab(widgets)
     ThemeTab(widgets)
+
     SaveTab(widgets)
 
     uistuff.keepWindowSize("preferencesdialog", widgets["preferences_dialog"])
@@ -72,14 +76,14 @@ class GeneralTab:
         # Give to uistuff.keeper
 
         for key in ("firstName", "secondName", "showEmt", "showEval",
-                    "autoPromote", "hideTabs", "closeAll", "faceToFace",
-                    "showCords", "showCaptured", "figuresInNotation",
-                    "fullAnimation", "moveAnimation", "noAnimation",
-                    "showFICSgameno"):
+                    "hideTabs", "closeAll", "faceToFace",
+                    "showCaptured", "figuresInNotation",
+                    "moveAnimation", "noAnimation",
+                    "autoRotate", "showFICSgameno"):
             uistuff.keep(widgets[key], key)
 
         # Options on by default
-        for key in ("autoRotate", "fullAnimation", "showBlunder"):
+        for key in ("autoPromote", "fullAnimation", "showBlunder", "showCords", "drawGrid"):
             uistuff.keep(widgets[key], key, first_value=True)
 
 # Hint initing
@@ -124,9 +128,9 @@ class HintTab:
         self.widgets = widgets
 
         # Options on by default
-        for key in ("opening_check", "endgame_check", "online_egtb_check",
-                    "analyzer_check", "inv_analyzer_check"):
-            uistuff.keep(widgets[key], key, first_value=True)
+        for key in ("opening_check", "endgame_check", "online_egtb_check", "inv_analyzer_check"):
+            uistuff.keep(widgets[key], key, first_value=False)
+        uistuff.keep(widgets["analyzer_check"], "analyzer_check", first_value=True)
 
         # Opening book
         default_path = os.path.join(addDataPrefix("pychess_book.bin"))
@@ -212,8 +216,14 @@ class HintTab:
 
         # Save, load and make analyze combos active
 
-        # Let Stockfish to be default analyzer in Windows installer
-        default = discoverer.getEngineN(-1).get("md5")
+        if sys.platform == "win32":
+            # Let Stockfish to be default analyzer in Windows installer
+            default = discoverer.getEngineN(-1).get("md5")
+        else:
+            engine = discoverer.getEngineByName("stockfish")
+            if engine is None:
+                engine = discoverer.getEngineN(-1)
+            default = engine.get("md5")
         conf.set("ana_combobox", conf.get("ana_combobox", default))
         conf.set("inv_ana_combobox", conf.get("inv_ana_combobox", default))
 
@@ -231,10 +241,8 @@ class HintTab:
                     for gmwidg in perspective.gamewidgets:
                         gmwidg.gamemodel.remove_analyzer(HINT)
 
-        self.widgets["analyzers_vbox"].set_sensitive(widgets[
-            "analyzer_check"].get_active())
-        self.widgets["analyzer_check"].connect_after("toggled",
-                                                     on_analyzer_check_toggled)
+        self.widgets["analyzers_vbox"].set_sensitive(widgets["analyzer_check"].get_active())
+        self.widgets["analyzer_check"].connect_after("toggled", on_analyzer_check_toggled)
 
         def on_invanalyzer_check_toggled(check):
             self.widgets["inv_analyzers_vbox"].set_sensitive(check.get_active())
@@ -249,10 +257,8 @@ class HintTab:
                     for gmwidg in perspective.gamewidgets:
                         gmwidg.gamemodel.remove_analyzer(SPY)
 
-        self.widgets["inv_analyzers_vbox"].set_sensitive(widgets[
-            "inv_analyzer_check"].get_active())
-        self.widgets["inv_analyzer_check"].connect_after(
-            "toggled", on_invanalyzer_check_toggled)
+        self.widgets["inv_analyzers_vbox"].set_sensitive(widgets["inv_analyzer_check"].get_active())
+        self.widgets["inv_analyzer_check"].connect_after("toggled", on_invanalyzer_check_toggled)
 
         # Give widgets to keeper
 
@@ -265,6 +271,8 @@ class HintTab:
             lambda combobox, value: anal_combo_set_value(combobox, value, "spy_mode", "inv_analyzer_check", SPY))
 
         uistuff.keep(self.widgets["max_analysis_spin"], "max_analysis_spin", first_value=3)
+        uistuff.keep(self.widgets["infinite_analysis"], "infinite_analysis", first_value=False)
+
 
 # Sound initing
 
@@ -464,8 +472,9 @@ class PanelTab:
     def __init__(self, widgets):
         # Put panels in trees
         self.widgets = widgets
-
-        from pychess.perspectives.games import sidePanels, dockLocation
+        persp = perspective_manager.get_perspective("games")
+        sidePanels = persp.sidePanels
+        dockLocation = persp.dockLocation
 
         saved_panels = []
         xmlOK = os.path.isfile(dockLocation)
@@ -548,9 +557,11 @@ class PanelTab:
         if active:
             leaf = persp.notebooks["board"].get_parent().get_parent()
             leaf.dock(persp.docks[name][1], EAST, persp.docks[name][0], name)
+            panel.menu_item.show()
         else:
             try:
                 persp.notebooks[name].get_parent().get_parent().undock(persp.notebooks[name])
+                panel.menu_item.hide()
             except AttributeError:
                 # A new panel appeared in the panels directory
                 leaf = persp.notebooks["board"].get_parent().get_parent()
@@ -582,6 +593,10 @@ class PanelTab:
 
 # Theme initing
 
+board_items = [(None, "colors only")]
+boards_path = addDataPrefix("boards")
+board_items += [(get_pixbuf(os.path.join(boards_path, b), 24), b[:-6]) for b in listdir(boards_path) if b.endswith("_d.png")]
+
 
 class ThemeTab:
     """ :Description: Allows the setting of various user specific chess
@@ -626,6 +641,24 @@ class ThemeTab:
 
         image_chooser_button.connect("file-set", select_new_image)
 
+        # Board style
+        uistuff.createCombo(widgets["board_style"], name="board_style")
+        data = [(item[0], item[1]) for item in board_items]
+
+        uistuff.createCombo(widgets["board_style"], data)
+        uistuff.keep(widgets["board_style"], "board_style", first_value=1)
+
+        # conf.set("board_style", conf.get("board_style", 1))
+
+        # Board frame
+        uistuff.createCombo(widgets["board_frame"], name="board_frame")
+        data = [(item[0], item[1]) for item in [(None, "no frame")] + board_items[1:]]
+
+        uistuff.createCombo(widgets["board_frame"], data)
+        uistuff.keep(widgets["board_frame"], "board_frame", first_value=1)
+
+        # conf.set("board_frame", conf.get("board_frame", 1))
+
         # Board Colours
 
         style_ctxt = widgets["main_window"].get_style_context()
@@ -727,8 +760,7 @@ class ThemeTab:
                 index = 0
             iconview.select_path(Gtk.TreePath(index, ))
 
-        uistuff.keep(widgets["pieceTheme"], "pieceTheme", _getActive,
-                     _setActive, "Chessicons")
+        uistuff.keep(widgets["pieceTheme"], "pieceTheme", _getActive, _setActive, "Chessicons")
 
     def discoverThemes(self):
         """ :Description: Finds all the different chess sets that are present
@@ -769,8 +801,7 @@ class SaveTab:
             widgets["autosave_grid"].set_property("sensitive", checkbox.get_active())
 
         conf.notify_add("autoSave", checkCallBack)
-        widgets["autoSave"].set_active(False)
-        uistuff.keep(widgets["autoSave"], "autoSave")
+        uistuff.keep(widgets["autoSave"], "autoSave", first_value=True)
         checkCallBack(_)
 
         default_path = os.path.expanduser("~")
@@ -806,4 +837,6 @@ class SaveTab:
 
         uistuff.keep(widgets["saveEmt"], "saveEmt")
         uistuff.keep(widgets["saveEval"], "saveEval")
-        uistuff.keep(widgets["saveOwnGames"], "saveOwnGames")
+        uistuff.keep(widgets["saveRatingChange"], "saveRatingChange")
+        uistuff.keep(widgets["indentPgn"], "indentPgn")
+        uistuff.keep(widgets["saveOwnGames"], "saveOwnGames", first_value=True)
